#version 330

in vec3 VS_normal_ws;
in vec3 VS_position_ws;

out vec4 final_color;

uniform vec4 light_position;
uniform vec3 light_ambient_color;
uniform vec3 light_diffuse_color;
uniform vec3 light_specular_color;
uniform vec3 eye_position;

uniform vec3 material_ambient_color;
uniform vec3 material_diffuse_color;
uniform vec3 material_specular_color;
uniform float material_shininess;

void main()
{
    // Direction from the current fragment to the light source.
    // Instead of using 'if (light_position.w == 0.0)', we use this. For
    // directional lights, w is 0.0, so the fragment position is not subtracted
    vec3 L = normalize(light_position.xyz - VS_position_ws * light_position.w);

    // Normal vector of the current fragment
    vec3 N = normalize(VS_normal_ws);

    // Direction from the current fragment to the camera
    vec3 Eye = normalize(eye_position - VS_position_ws);
    // Half vector (between Eye and L)
    vec3 H = normalize(Eye + L);

    // Distance of the current fragment from the light source; 0 if directional
    float d = distance(light_position.xyz, VS_position_ws) * light_position.w;
    // Attenuation factor
    float atten = 1.0 / (1.0 + 0.1 * d + 0.01 * d * d);

    // Diffuse light intensity
    float Idiff = max(dot(L, N), 0.0) * atten;
    // Specular light intensity
    float Ispec = pow(max(dot(N, H), 0.0), material_shininess) * Idiff;

    // Final light
    vec3 light = light_ambient_color * material_ambient_color
                 + Idiff * light_diffuse_color * material_diffuse_color
                 + Ispec * light_specular_color * material_specular_color;

    final_color = vec4(light, 1.0);
}
